import { sink, } from "@reactive-js/async-enumerable";
import { writeHttpResponseHeaders, parseHttpRequestFromHeaders, } from "@reactive-js/http";
import { createBufferStreamFromReadable, createBufferStreamSinkFromWritable, } from "@reactive-js/node";
import { await_, catchError, onNotify, subscribe, empty, compute, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
import { AbstractDisposable } from "@reactive-js/disposable";
import { isSome } from "@reactive-js/option";
class RequestBody extends AbstractDisposable {
    constructor(req) {
        super();
        this.req = req;
        this.consumed = false;
        this.add(_ => {
            req.removeAllListeners();
            req.once("error", () => { });
            req.once("close", () => {
                req.removeAllListeners();
            });
            req.destroy();
        });
        const onError = (cause) => {
            this.dispose({ cause });
        };
        req.on("error", onError);
    }
    enumerateAsync(scheduler, replayCount) {
        if (this.consumed) {
            throw new Error("Request body already consumed");
        }
        this.consumed = true;
        const sink = createBufferStreamFromReadable(() => this.req)
            .enumerateAsync(scheduler, replayCount)
            .add(this);
        this.add(sink);
        return sink;
    }
}
class ResponseBody extends AbstractDisposable {
    constructor(resp) {
        super();
        this.resp = resp;
        this.consumed = false;
        this.add(_ => {
            resp.removeAllListeners();
            resp.once("error", () => { });
            resp.once("close", () => {
                resp.removeAllListeners();
            });
            resp.destroy();
        });
        const onError = (cause) => {
            this.dispose({ cause });
        };
        resp.on("error", onError);
    }
    enumerateAsync(scheduler, replayCount) {
        if (this.consumed) {
            throw new Error("Response body already consumed");
        }
        this.consumed = true;
        const sink = createBufferStreamSinkFromWritable(() => this.resp, true)
            .enumerateAsync(scheduler, replayCount)
            .add(this);
        this.add(sink);
        return sink;
    }
}
const writeResponseMessage = (responseBody) => (response) => {
    responseBody.resp.statusCode = response.statusCode;
    writeHttpResponseHeaders(response, (header, value) => responseBody.resp.setHeader(header, value));
};
const writeResponseBody = (responseBody) => ({ body, contentInfo, }) => isSome(contentInfo) ? sink(body, responseBody) : empty();
const defaultOnError = (_) => empty();
export const createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (requestBody, responseBody) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = requestBody.req;
        const isTransportSecure = (_a = requestBody.req.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        return pipe(() => parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body: requestBody,
        }), compute, await_(handler), onNotify(writeResponseMessage(responseBody)), await_(writeResponseBody(responseBody)), catchError(onError));
    };
    return (req, resp) => {
        const requestBody = new RequestBody(req);
        const responseBody = new ResponseBody(resp);
        responseBody.add(pipe(handleRequest(requestBody, responseBody), subscribe(scheduler)).add(requestBody));
    };
};
//# sourceMappingURL=httpRequestListener.js.map